library fluttercustomtoast;
import 'dart:ui';
import 'package:flutter/material.dart';

/// A Calculator.
class Calculator {
  /// Returns [value] plus 1.
  int addOne(int value) => value + 1;
}

class CustomToast {
  static OverlayEntry _overlayEntry; //toast靠它加到屏幕上
  static bool _showing = false; //toast是否正在showing
  static DateTime _startedTime; //开启一个新toast的当前时间，用于对比是否已经展示了足够时间
  static String _msg;
  static void toast(
      BuildContext context,
      String msg,
      double top,
      double fillet,
      double fontSize,
      Color backgroundColor,
      Color textColor,
      ) async {
    assert(msg != null);
    _msg = msg;
    _startedTime = DateTime.now();
    //获取OverlayState
    OverlayState overlayState = Overlay.of(context);
    _showing = true;
    if (_overlayEntry == null) {
      _overlayEntry = OverlayEntry(
          builder: (BuildContext context) => Positioned(
            //top值，可以改变这个值来改变toast在屏幕中的位置
            top: top,
            child: Container(
                alignment: Alignment.center,
                width: MediaQuery.of(context).size.width,
//                height: height,
                child: Padding(
                  padding: EdgeInsets.symmetric(horizontal: 0.0, vertical: 10),
                  child: AnimatedOpacity(
                    opacity: _showing ? 1.0 : 0.0, //目标透明度
                    duration: _showing
                        ? Duration(milliseconds: 100)
                        : Duration(milliseconds: 400),
                    child: _buildToastWidget(fillet, fontSize, backgroundColor, textColor),
                  ),
                )),
          ));
      overlayState.insert(_overlayEntry);
    } else {
      //重新绘制UI，类似setState
      _overlayEntry.markNeedsBuild();
    }
    await Future.delayed(Duration(milliseconds: 2000)); //等待两秒

    //2秒后 到底消失不消失
    if (DateTime.now().difference(_startedTime).inMilliseconds >= 2000) {
      _showing = false;
      _overlayEntry.markNeedsBuild();
    }
  }

  //toast绘制
  static _buildToastWidget(double fillet,
      double fontSize,
      Color backgroundColor,
      Color textColor) {
    return Center(
      child: Container(
        decoration: BoxDecoration(
            color: backgroundColor,
            borderRadius: BorderRadius.circular(fillet)
        ),
//        color: Colors.black26,
        child: Padding(
          padding: EdgeInsets.symmetric(horizontal: 10.0, vertical: 10),
          child: Text(
            _msg,
            style: TextStyle(
                fontSize: fontSize,
                fontWeight: FontWeight.w300,
                color: textColor,
                decoration: TextDecoration.none
            ),
          ),
        ),
      ),
    );
  }

  static void showToast({
    @required BuildContext context,
    @required String msg,
    double top = 60,
    double height = 80,
    double fillet = 40,
    double fontSize = 14,
    Color backgroundColor = Colors.black26,
    Color textColor = Colors.white,
  }) async {
    return CustomToast.toast(context, msg, top, fillet, fontSize, backgroundColor, textColor);
  }
}